package mvd.jester.model;

import java.util.Set;
import org.jgrapht.experimental.dag.DirectedAcyclicGraph;
import org.jgrapht.graph.DefaultEdge;
import mvd.jester.utils.DagUtils;

public class DagTask extends Task {

    private DirectedAcyclicGraph<Job, DefaultEdge> jobDag;
    private final Set<Segment> workloadDistribution;

    public DagTask(final DirectedAcyclicGraph<Job, DefaultEdge> jobDag, final long period,
            final long numberOfThreads) {
        super(period, period, DagUtils.calculateWorkload(jobDag),
                DagUtils.calculateCriticalPath(jobDag), numberOfThreads);
        this.jobDag = jobDag;
        this.workloadDistribution =
                DagUtils.calculateWorkloadDistribution(this.jobDag, this.criticalPath);
    }

    public DagTask(final DirectedAcyclicGraph<Job, DefaultEdge> jobDag, final long period,
            final long deadline, final long numberOfThreads) {
        super(period, deadline, DagUtils.calculateWorkload(jobDag),
                DagUtils.calculateCriticalPath(jobDag), numberOfThreads);
        this.jobDag = jobDag;
        this.workloadDistribution =
                DagUtils.calculateWorkloadDistribution(this.jobDag, this.criticalPath);
    }

    public DagTask(final DirectedAcyclicGraph<Job, DefaultEdge> jobDag, final long period,
            final Set<Segment> workloadDistribution, final long numberOfThreads) {
        super(period, period, DagUtils.calculateWorkload(jobDag),
                DagUtils.calculateCriticalPath(jobDag), numberOfThreads);
        this.jobDag = jobDag;
        this.workloadDistribution = workloadDistribution;
    }


    /**
     * @return the jobDag
     */
    public DirectedAcyclicGraph<Job, DefaultEdge> getJobDag() {
        return jobDag;
    }

    /**
     * @param jobDag the jobDag to set
     */
    public void setJobDag(final DirectedAcyclicGraph<Job, DefaultEdge> jobDag) {
        this.jobDag = jobDag;
    }

    /**
     * @return the workloadDistribution
     */
    @Override
    public Set<Segment> getWorkloadDistribution() {
        return workloadDistribution;
    }

    @Override
    public long getMaximumParallelism() {
        long max = 0;
        for (final Segment s : workloadDistribution) {
            if (max < s.getHeight()) {
                max = s.getHeight();
            }
        }
        return max;
    }

}
