package mvd.jester.model;

import java.util.Set;

/**
 * Task
 */
public class SynchronousTask extends Task {

    private final Set<Segment> segments;

    public SynchronousTask(final long period, final long deadline, final long numberOfThreads,
            final Set<Segment> segments) {
        super(period, deadline, SynchronousUtils.calculateWorkload(segments),
                SynchronousUtils.calculateCriticalPath(segments), numberOfThreads);
        this.segments = segments;
    }

    public SynchronousTask(final Set<Segment> segments, final long period,
            final long numberOfThreads) {
        this(period, period, numberOfThreads, segments);
    }

    public double getUtilization() {
        return (double) workload / period;
    }

    /**
     * @return the deadline
     */
    public long getDeadline() {
        return deadline;
    }

    /**
     * @return the period
     */
    public long getPeriod() {
        return period;
    }

    /**
     * @return the segments
     */
    public Set<Segment> getWorkloadDistribution() {
        return segments;
    }

    /**
     * @return the maximumWcet
     */
    public long getWorkload() {
        return workload;
    }

    /**
     * @return the criticalPath
     */
    public long getCriticalPath() {
        return criticalPath;
    }

    @Override
    public long getMaximumParallelism() {
        long max = 0;
        for (final Segment s : segments) {
            if (max < s.getHeight()) {
                max = s.getHeight();
            }
        }
        return max;
    }

    @Override
    public long getNumberOfThreads() {
        return numberOfThreads;
    }

    public static class SynchronousUtils {
        public static long calculateWorkload(final Set<Segment> segments) {
            long workload = 0;
            for (final Segment s : segments) {
                workload += s.getWidth() * s.getHeight();
            }
            return workload;
        }


        public static long calculateCriticalPath(final Set<Segment> segments) {
            long criticalPath = 0;
            for (final Segment s : segments) {
                criticalPath += s.getWidth();
            }

            return criticalPath;
        }
    }
}
