package mvd.jester.tests;

import mvd.jester.info.SchedulingInfo;
import mvd.jester.info.SchedulingInfo.Feasiblity;
import mvd.jester.model.DagTask;
import mvd.jester.model.SortedTaskSet;
import mvd.jester.model.SystemManagerInterface;
import mvd.jester.priority.EarliestDeadlineFirst;
import mvd.jester.priority.PriorityManager;

public class DongLiu extends AbstractTest<DagTask> {

    private final EarliestDeadlineFirst priorityManager;

    public DongLiu(SystemManagerInterface manager) {
        super(manager);
        this.priorityManager = new EarliestDeadlineFirst();
    }

    @Override
    public SchedulingInfo runSchedulabilityCheck(SortedTaskSet<DagTask> tasks) {
        final long numberOfProcessors = manager.getNumberOfProcessors();
        for (DagTask task : tasks) {
            double sumOfWorkloads = 0;
            for (DagTask t : tasks) {
                sumOfWorkloads += getAverageWorkload(t, task);
            }
            final double cpUtilization = (double) task.getCriticalPath() / task.getPeriod();
            if (!(sumOfWorkloads <= numberOfProcessors
                    - ((numberOfProcessors - 1) * cpUtilization))) {
                return new SchedulingInfo(Feasiblity.FAILED);
            }
        }

        return new SchedulingInfo(Feasiblity.SUCCEEDED);
    }

    private double getAverageWorkload(DagTask t, DagTask task) {
        final double cpUtilization = (double) task.getCriticalPath() / task.getPeriod();
        final double taskUtilization = t.getUtilization();
        if (cpUtilization >= taskUtilization) {
            return taskUtilization;
        } else {
            return taskUtilization + ((double) t.getWorkload() - cpUtilization * t.getPeriod())
                    / (task.getPeriod());
        }
    }

    @Override
    public PriorityManager getPriorityManager() {
        return priorityManager;
    }

    @Override
    public String getName() {
        return "DongLiu";
    }

}
