package mvd.jester;

import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.ThreadLocalRandom;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import mvd.jester.info.SchedulingInfo;
import mvd.jester.priority.PriorityManager;

/**
 * TestResultCollector
 */
public class TestResultCollector {

    @Test
    @DisplayName("Test if getters and setters work correctly.")
    void testGettersAndSetters() {
        final PriorityManager pm = mock(PriorityManager.class);
        final String pmName = "PriorityManagerName";
        when(pm.getName()).thenReturn(pmName);

        final TypeInterface ti = mock(TypeInterface.class);
        final String tiName = "TypeInterfaceName";
        when(ti.getName()).thenReturn(tiName);

        final ResultCollector<TypeInterface> rc1 = new ResultCollector<>(pm, ti);
        final ResultCollector<TypeInterface> rc2 = new ResultCollector<>(pm, ti);

        assertTrue(rc1.getPriorityManager() == pm);
        assertTrue(rc1.getAbstractValue() == ti);
        assertTrue(rc1.getName().equals(tiName + "_" + pmName));
        assertTrue(rc1.compareTo(rc2) == 0);
    }

    @Test
    @DisplayName("Check if adding SchedulingInfos works correctly.")
    void testSchedulingInfos() {
        final TypeInterface ti = mock(TypeInterface.class);
        final PriorityManager pm = mock(PriorityManager.class);

        final ResultCollector<TypeInterface> rc = new ResultCollector<>(pm, ti);

        final Set<SchedulingInfo> schedulingInfos = new HashSet<>();

        long numberOfSchedulingInfos = ThreadLocalRandom.current().nextLong(5, 10);
        for (int i = 0; i < numberOfSchedulingInfos; ++i) {
            SchedulingInfo si = mock(SchedulingInfo.class);
            schedulingInfos.add(si);
            assertTrue(rc.addResult(si));
            assertFalse(rc.addResult(si));
        }

        assertTrue(rc.getResults().size() == numberOfSchedulingInfos);
        assertTrue(rc.getResults().equals(schedulingInfos));
        assertTrue(rc.addResult(null));
    }
}
